@@ -2,7 +2,9 @@ from __future__ import absolute_import, division, print_function
 
 from collections import OrderedDict
 from json.decoder import JSONDecodeError
-from astropy.table import Table
+
+import rdflib
+import typing
 from astropy.coordinates import Angle
 
 # NOTE gw is optional for now
@@ -13,7 +15,8 @@ try:
 except ModuleNotFoundError:
     pass
 
-from .data_products import NumpyDataProduct, BinaryData, ApiCatalog, GWContoursDataProduct
+from .data_products import NumpyDataProduct, BinaryData, ApiCatalog, GWContoursDataProduct, PictureProduct
+from oda_api.token import TokenLocation
 
 from builtins import (bytes, str, open, super, range,
                       zip, round, input, int, pow, object, map, zip)
@@ -48,7 +51,7 @@ from itertools import cycle
 import numpy as np
 import traceback
 from jsonschema import validate as validate_json
-from typing import Union
+from typing import Union, Tuple
 
 import oda_api.token
 import oda_api.misc_helpers
@@ -196,6 +199,27 @@ class DispatcherAPI:
     # but in desirable way
     token_discovery_methods = None
 
+    _known_sites_dict = None
+
+    @property
+    def known_sites_dict(self):
+        if self._known_sites_dict is None:
+            self._known_sites_dict = {}
+
+            G = rdflib.Graph()
+            G.parse("https://odahub.io/oda-sites.ttl")
+            for site, url in G.subject_objects(rdflib.URIRef("http://odahub.io/ontology#APIURL")):             
+                self._known_sites_dict[site.split("#")[1]] = str(url)
+
+                for alias in G.objects(site, rdflib.URIRef("http://www.w3.org/2000/01/rdf-schema#label")):
+                    self._known_sites_dict[str(alias)] = str(url)
+        
+        return self._known_sites_dict
+
+    def setup_loggers(self):
+        self.logger = logger.getChild(self.__class__.__name__.lower())
+        self.progress_logger = self.logger.getChild("progress")
+
     def __init__(self,
                  instrument='mock',
                  url=None,
@@ -209,8 +233,22 @@ class DispatcherAPI:
                  session_id=None,
                  ):
 
+        self.setup_loggers()
+
         if url is None:
-            url = "https://www.astro.unige.ch/mmoda/dispatch-data"
+            if 'unige-production' not in self.known_sites_dict:
+                url = "https://www.astro.unige.ch/mmoda/dispatch-data"
+            else:
+                url = self.known_sites_dict['unige-production']
+        else:
+            if not url.startswith("http://") and not url.startswith("https://"):
+                self.logger.info('url %s is not of http(s) schema, trying to interpretting url as an alias', url)
+                if url in self.known_sites_dict:                                        
+                    self.logger.info('url %s interpretted an alias for %s', url, self.known_sites_dict[url])
+                    url = self.known_sites_dict[url]
+                else:
+                    logger.debug(f'url %s does not match http(s) schema and is not one of the aliases (%s)', url, list(self.known_sites_dict))                    
+
 
         if host is not None:
             msg = '\n'
@@ -241,10 +279,7 @@ class DispatcherAPI:
 
         if session_id is not None:
             self._session_id = session_id
-
-        self.logger = logger.getChild(self.__class__.__name__.lower())
-        self.progress_logger = self.logger.getChild("progress")
-
+        
         self._carriage_return_progress = False
 
         self.run_analysis_handle = run_analysis_handle
@@ -297,6 +332,35 @@ class DispatcherAPI:
         else:
             raise RuntimeError(r.text)
 
+    def refresh_token(self,
+                      token_to_refresh=None,
+                      write_token=False,
+                      token_write_methods: Union[Tuple[TokenLocation, ...], TokenLocation] = (TokenLocation.ODA_ENV_VAR,
+                                                                                         TokenLocation.FILE_CUR_DIR),
+                      discard_discovered_token=False):
+        if token_to_refresh is None:
+            token_to_refresh = oda_api.token.discover_token()
+        if token_to_refresh is not None and token_to_refresh != '':
+            params = dict(token=token_to_refresh,
+                          query_status='new')
+
+            r = requests.get(os.path.join(self.url, 'refresh_token'),
+                             params=params)
+
+            if r.status_code == 200:
+                refreshed_token = r.text
+                if write_token:
+                    oda_api.token.rewrite_token(refreshed_token,
+                                                old_token=token_to_refresh,
+                                                token_write_methods=token_write_methods,
+                                                discard_discovered_token=discard_discovered_token)
+
+                return refreshed_token
+            else:
+                raise RuntimeError(r.text)
+        else:
+            raise RuntimeError("unable to refresh the token with any known method")
+
     def set_custom_progress_formatter(self, F):
         self.custom_progress_formatter = F
 
@@ -754,6 +818,14 @@ class DispatcherAPI:
                           res_json['exit_status']['error_message'])
         self.logger.error('Remote server debug_message-> %s',
                           res_json['exit_status']['debug_message'])
+        
+    def show_status_comments(self, res_json):
+        if res_json['exit_status']['comment']:
+            print(res_json['exit_status']['comment'])
+        # TODO: warning field is not currently consistently used 
+        #       could be enabled in the future (add test then!)
+        # if res_json['exit_status']['warning']:
+        #     self.logger.warning(res_json['exit_status']['warning'])
 
     def dig_list(self, b, only_prod=False):
         if isinstance(b, (set, tuple, list)):
@@ -875,10 +947,10 @@ class DispatcherAPI:
             f"{C.GREY}last request completed in {self.last_request_t_complete - self.last_request_t0} seconds{C.NC}")
 
     def get_list_terms_gallery(self,
-                               group: str = None,
-                               parent: str = None,
-                               parent_id: str = None,
-                               token: str = None
+                               group: typing.Optional[str] = None,
+                               parent: typing.Optional[str] = None,
+                               parent_id: typing.Optional[str] = None,
+                               token: typing.Optional[str] = None
                                ):
         logger.debug("Getting the list of available instruments on the gallery")
         params = {
@@ -902,11 +974,11 @@ class DispatcherAPI:
         return response_json
 
     def post_data_product_to_gallery(self,
-                                     product_title: str = None,
-                                     observation_id: str = None,
-                                     gallery_image_path: str = None,
+                                     product_title: typing.Optional[str] = None,
+                                     observation_id: typing.Optional[str] = None,
+                                     gallery_image_path: typing.Optional[str] = None,
                                      fits_file_path=None,
-                                     token: str = None,
+                                     token: typing.Optional[str] = None,
                                      insert_new_source: bool = False,
                                      validate_source: bool = False,
                                      force_insert_not_valid_new_source: bool = False,
@@ -1020,8 +1092,8 @@ class DispatcherAPI:
         return response_json
 
     def resolve_source(self,
-                       src_name: str = None,
-                       token: str = None):
+                       src_name: typing.Optional[str] = None,
+                       token: typing.Optional[str] = None):
         resolved_obj = None
         if src_name is not None and src_name != '':
             params = {
@@ -1044,7 +1116,7 @@ class DispatcherAPI:
         return resolved_obj
 
     def check_gallery_data_product_policy(self,
-                                          token: str = None,
+                                          token: typing.Optional[str] = None,
                                           **kwargs):
         product_type = kwargs.get('product_type', None)
         if product_type is not None and product_type != '':
@@ -1075,7 +1147,7 @@ class DispatcherAPI:
 
         return True
 
-    def check_missing_parameters_data_product(self, response, token: str = None, **kwargs):
+    def check_missing_parameters_data_product(self, response, token: typing.Optional[str] = None, **kwargs):
         missing_instrument = True
         instrument_used = None
         missing_product_type = True
@@ -1169,8 +1241,8 @@ class DispatcherAPI:
                         msg += '\n'
                         msg += '%s' % valid_names
                         msg += '\n'
-                        msg += 'this will throw an error in a future version \n'
-                        msg += 'and might break the current request!\n '
+                        # msg += 'this will throw an error in a future version \n'
+                        # msg += 'and might break the current request!\n '
                         msg += '----------------------------------------------------------------------------\n'
                         warnings.warn(msg)
 
@@ -1199,6 +1271,8 @@ class DispatcherAPI:
             raise RuntimeError(
                 "not failed, not, but complete? programming error for client!")
 
+        self.show_status_comments(res_json)
+        
         d = DataCollection.from_response_json(
             res_json, instrument, product)
 
@@ -1351,7 +1425,7 @@ class DataCollection(object):
         if 'binary_data_product_list' in res_json['products'].keys():
             data.extend([BinaryData().decode(d)
                          for d in res_json['products']['binary_data_product_list']])
-
+        
         if 'catalog' in res_json['products'].keys():
             data.append(ApiCatalog(
                 res_json['products']['catalog'], name='dispatcher_catalog'))
@@ -1364,6 +1438,14 @@ class DataCollection(object):
             data.extend([ascii.read(table_binary)
                          for table_binary in res_json['products']['astropy_table_product_binary_list']])
 
+        if 'binary_image_product_list' in res_json['products'].keys():
+            data.extend([PictureProduct.decode(bin_image_data)
+                         for bin_image_data in res_json['products']['binary_image_product_list']])
+        
+        if 'text_product_list' in res_json['products'].keys():
+            data.extend([text_data
+                         for text_data in res_json['products']['text_product_list']])
+            
         if 'gw_strain_product_list' in res_json['products'].keys():
             data.extend([TimeSeries(strain_data['value'],
                                     name=strain_data['name'],
