@@ -75,8 +75,7 @@ def test_get(dispatcher_live_fixture, caplog):
     runner = CliRunner()
     result = runner.invoke(cli.cli, ['-u', dispatcher_live_fixture, 'get', '-i', 'empty'], obj={})
     assert result.exit_code == 0
-
-    assert "'prod_dict': {'dummy': 'empty_parameters_dummy_query', 'failing': '" in caplog.text
+    assert "'prod_dict': {'dummy': 'empty_parameters_dummy_query', 'echo': 'echo_parameters_dummy_query', 'failing': '" in caplog.text
 
     runner = CliRunner()
     result = runner.invoke(cli.cli, ['-u', dispatcher_live_fixture, '--no-wait', 'get', '-i', 'empty', '-p', 'dummy', '-a', 'product_type=Dummy'], obj={})
