@@ -1,9 +1,12 @@
 import pathlib
-
 import pytest
 
+from cdci_data_analysis.pytest_fixtures import DispatcherJobState
+
+
 @pytest.mark.live
 def test_plot_tools_notebook(request):
+    DispatcherJobState.remove_scratch_folders()
     import papermill as pm
 
     in_nb = pathlib.Path(request.fspath.dirname) / "../doc/source/user_guide/Show_and_Save_Products.ipynb"
